#include "CSerial.h"

CComm::CComm()
{
	mHThread = NULL;
	mMutex = CreateMutex( NULL, TRUE, NULL );
	mBuffer.dwRp = 0;
	mBuffer.dwWp = 0;
	mBuffer.dwMax = sizeof(mBuffer.data) - 1;
}

CComm::~CComm()
{
	CloseHandle( mMutex );
	mMutex = NULL;
}

bool CComm::Open( void* p )
{
	return false;
}

void CComm::Close()
{
}

DWORD CComm::Receive()
{
	return S_OK;
}

DWORD CComm::Transmit( const BYTE* pBuf, DWORD dwSize )
{
	return 0;
}

DWORD CComm::GetReceiveData( BYTE* pBuf )
{
	return ReadBuffer( pBuf );
}

DWORD CComm::GetReceiveDataSize()
{
	return (mBuffer.dwWp - mBuffer.dwRp) & mBuffer.dwMax;
}

void CComm::ClearReceiveBuffer()
{
	mBuffer.dwRp = 0;
	mBuffer.dwWp = 0;
}

void CComm::WriteBuffer( BYTE* pData, DWORD dwSize )
{
	WaitForSingleObject( mMutex, 0 );

	BYTE *pdst, *psrc;
	DWORD w;

	if( (mBuffer.dwWp + dwSize) <= sizeof(mBuffer.data) )
	{
		pdst = &mBuffer.data[mBuffer.dwWp];
		psrc = pData;
		memcpy(pdst, psrc, dwSize);
	}
	else
	{
		pdst = &mBuffer.data[mBuffer.dwWp];
		psrc = pData;
		w = sizeof(mBuffer.data) - mBuffer.dwWp;
		memcpy( pdst, psrc, w );

		pdst = &mBuffer.data[0];
		psrc = pData + w;
		w = dwSize - w;
		memcpy( pdst, psrc, w );
	}
	mBuffer.dwWp += dwSize;
	mBuffer.dwWp &= mBuffer.dwMax;

	ReleaseMutex( mMutex );
}

DWORD CComm::ReadBuffer( BYTE* pbuf )
{
	WaitForSingleObject( mMutex, 0 );

	BYTE *pdst, *psrc;
	DWORD r, size, len;

	r = mBuffer.dwRp;
	size = (mBuffer.dwWp - r) & mBuffer.dwMax;

	if( size != 0 )
	{
		if( (r + size) <= sizeof(mBuffer.data) )
		{
			pdst = pbuf;
			psrc = &mBuffer.data[r];
			len = size;
			memcpy( pdst, psrc, len );
		}
		else
		{
			pdst = pbuf;
			psrc = &mBuffer.data[r];
			len = sizeof(mBuffer.data) - r;
			memcpy( pdst, psrc, len );

			pdst = pbuf + len;
			psrc = &mBuffer.data[0];
			len = size - len;
			memcpy( pdst, psrc, len );
		}

		mBuffer.dwRp += size;
		mBuffer.dwRp &= mBuffer.dwMax;
	}

	ReleaseMutex(mMutex);

	return size;
}
