#include "CSerial.h"
#include <process.h>

static unsigned __stdcall ThreadFunc( void *p )
{
	CSerial *pp = (CSerial*)p;
	return pp->Receive();
}

CSerial::CSerial():
	mHComPort(NULL),
	fTerminate(false)
{
}

CSerial::~CSerial()
{
}

bool CSerial::Open( void* p )
{
	int comNo = *(int*)p;
	// Open com device
	char str[32];
	wsprintf( str, "\\\\.\\COM%d", comNo );
	mHComPort = CreateFile(
		str,
		GENERIC_READ | GENERIC_WRITE,
		0,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
		NULL
	);
	if( mHComPort == INVALID_HANDLE_VALUE ) return false;

	// A character was received and placed in the input buffer
	if( !SetCommMask( mHComPort, EV_RXCHAR ) )
	{
		CloseHandle( mHComPort );
		return false;
	}

	// Setup device buffers
	if( !SetupComm( mHComPort, 4096, 4096 ) )
	{
		CloseHandle( mHComPort );
		return false;
	}

	// Purge any information in the buffer
	if( !PurgeComm( mHComPort, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR ) )
	{
		CloseHandle( mHComPort );
		return false;
	}

	// Sets the time-out parameters
	COMMTIMEOUTS timeout;
	timeout.ReadIntervalTimeout = MAXDWORD;
	timeout.ReadTotalTimeoutMultiplier = 0;
	timeout.ReadTotalTimeoutConstant = 1000;
	timeout.WriteTotalTimeoutMultiplier = 10;
	timeout.WriteTotalTimeoutConstant = 1000;
	if( !SetCommTimeouts( mHComPort, &timeout ) )
	{
		CloseHandle( mHComPort );
		return false;
	}

	// Configures a communications device
	DCB dcb;
	if( !GetCommState( mHComPort, &dcb ) )
	{
		CloseHandle( mHComPort );
		return false;
	}
	dcb.BaudRate = 115200;
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.fParity = FALSE;
	dcb.StopBits = ONESTOPBIT;
	dcb.fOutxCtsFlow = TRUE;
	dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;
	dcb.fOutX = FALSE;
	dcb.fInX = FALSE;
	if( !SetCommState( mHComPort, &dcb ) )
	{
		CloseHandle( mHComPort );
		return false;
	}

	// Create a recieve thread
	fTerminate = false;
	mHThread = (HANDLE)_beginthreadex(
		NULL,
		0,
		ThreadFunc,
		this,
		0,
		(unsigned int*)&mThreadId
	);

	if( mHThread == 0 )
	{
		CloseHandle( mHComPort );
		return false;
	}

	return true;
}

void CSerial::Close()
{
	fTerminate = true;

	if( mHComPort != NULL )
	{
		// Disables all events
		SetCommMask( mHComPort, 0 );
	}

	// Retrieves the termination status of the thread
	DWORD dwExCode;
	GetExitCodeThread( mHThread , &dwExCode );
	if( dwExCode == STILL_ACTIVE )
	{
		// Wait until the thread is over
		WaitForSingleObject( mHThread, INFINITE );
		CloseHandle( mHThread );
	}


	if( mHComPort != NULL )
	{
		// Clears the input and output buffer
		PurgeComm( mHComPort, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR );
		CloseHandle( mHComPort );
	}
}

DWORD CSerial::Receive()
{
	COMSTAT ComStat;
	DWORD dwEvtMask, dwCount, dwRead;
	BYTE buffer[RECV_BUFF_SIZE];
	DWORD dwErrors;
	OVERLAPPED ol = {0};

	ol.hEvent = CreateEvent( NULL, TRUE, FALSE, NULL );
	if( ol.hEvent == NULL ) return S_FALSE;

	while( !fTerminate )
	{
		dwEvtMask = 0;

		// Wait for data receive event to occur
		WaitCommEvent( mHComPort, &dwEvtMask, NULL );

		if( dwEvtMask & EV_RXCHAR )
		{
			ClearCommError( mHComPort, &dwErrors, &ComStat );
			dwCount = ComStat.cbInQue;
			if( dwCount > 0 )
			{
				// Read receive data
				if( ReadFile( mHComPort, buffer, dwCount, &dwRead, &ol ) )
				{
					WriteBuffer( buffer, dwRead );
				}
				else
				{
					return S_FALSE;
				}
			}
		}
	}

	return S_OK;
}

DWORD CSerial::Transmit( const BYTE* pBuf, DWORD dwSize )
{
	DWORD w, index = 0;
	OVERLAPPED ol = {0};

	ol.hEvent = CreateEvent( NULL, TRUE, FALSE, NULL );
	if( ol.hEvent == NULL ) return S_FALSE;

	while( dwSize > 0 )
	{
		WriteFile( mHComPort, pBuf + index, dwSize, &w, &ol );
		index += w;
		if( !w )
		{
			return 0;
		}
		dwSize -= w;
	}

	return w;
}
